/**
 * Copyright (C) 2010 Cloudfarming <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.sensor.greenseeker;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import nl.cloudfarming.client.logging.AppLogFactory;
import nl.cloudfarming.client.logging.AppLogger;
import nl.cloudfarming.client.model.SensorData;
import nl.cloudfarming.client.model.SensorMetadata;
import nl.cloudfarming.client.sensor.SensorProcessor;
import nl.cloudfarming.client.sensor.SensorEventKey;
import nl.cloudfarming.eventbus.BaseGuiEventProducer;
import org.netbeans.api.progress.ProgressHandle;
import org.netbeans.api.progress.ProgressHandleFactory;
import org.openide.util.NbBundle;

/**
 * GreenSeeker File processor abstract class. Each inheriting GreenSeeker file processor implements its own parseLine method and exposes header-length and record length through getter-methods.
 * @author Gerben Feenstra
 */
public abstract class GreenseekerFileProcessor extends SensorProcessor<GreenseekerSensorData> {

    // Messages
    private static final String LOGKEY_FAILED_TO_READ_FILE = NbBundle.getMessage(GreenseekerFileProcessor.class, "sensor.common.fileprocessor.logerror.failed_to_read_file");
    private static final String LOGKEY_PROCESSING_SUMMARY_PROCESSEDMSG = NbBundle.getMessage(GreenseekerFileProcessor.class, "sensor.common.fileprocessor.loginfo.nr_of_processed");
    private static final String LOGKEY_PROCESSING_PARTIALLY_PROCESSEDMSG = NbBundle.getMessage(GreenseekerFileProcessor.class, "sensor.common.fileprocessor.loginfo.partiallyprocessed");
    private static final String LOGKEY_PROCESSING_SUMMARY_ERRORMSG = NbBundle.getMessage(GreenseekerFileProcessor.class, "sensor.common.fileprocessor.logerror.nr_of_records_failed");
    private static final String LOGKEY_PROCESSING_START = NbBundle.getMessage(GreenseekerFileProcessor.class, "sensor.common.fileprocessor.loginfo.process_start");
    // Constant int values
    private static final int BULK_COMMIT_AMOUNT = 100; // How many record should be gathered before doing a bulk-commit?
    private static final int PROGRESS_REPORT_INTERVAL = 5000; // How many record should be processed before printing a progress message in the log
    // Other variable declarations
    private static final AppLogger LOG = AppLogFactory.getLogger(GreenseekerFileProcessor.class);
    private final ProgressHandle progressHandle;
    private final EventProducer eventProducer = new EventProducer();
    protected File fileToProcess;

    /**
     * Create a GreenseekerFileProcessor instance to process the specified GreenSeeker sensor file.
     *
     * @param toProcess The GreenSeeker sensor file to be processed
     */
    public GreenseekerFileProcessor(File fileToProcess) {
        assert fileToProcess != null;
        this.fileToProcess = fileToProcess;
        this.progressHandle = ProgressHandleFactory.createHandle(NbBundle.getMessage(GreenseekerFileProcessor.class, "sensor.common.fileprocessor.loginfo.process_description", fileToProcess.getName()));
    }

    abstract protected GreenseekerSensorData parseLine(String payload) throws GreenseekerParseException;

    abstract protected List<String> parseFile(File sensorFile) throws FileNotFoundException, IOException;

    /**
     * Generic GreenSeeker method to process the specified file
     */
    @Override
    public List<SensorData> process() {
        //FIXME: re implement
        try {
            int success = 0;
            int fail = 0;
            int progress = 0;

            LOG.info(LOGKEY_PROCESSING_START, getLogDisplayName(), fileToProcess.getName());

            // Create Datafile
//            dataFile = createSensorDataFile(fileToProcess);

            List<String> recordList = parseFile(fileToProcess);

            List<GreenseekerSensorData> persistBuffer = new ArrayList<GreenseekerSensorData>();

            // Start progressbar
            this.progressHandle.start(recordList.size());

            // Start processing lines in list
            for (String line : recordList) {
                try {
                    //persistBuffer.add(parseLine(line, dataFile));

                    // Persist
                    if (success++ % BULK_COMMIT_AMOUNT == 0) {
                        persistBuffer.clear();
                    }
//                } catch (GreenseekerParseException ex) {
//                    LOG.trace(ex);
//                    fail++;
                } catch (Exception ex) {
                    LOG.trace(ex);
                    fail = fail + BULK_COMMIT_AMOUNT;
                    persistBuffer.clear();
                }

                progressHandle.progress(++progress);

                // Log partial progress at intervals
                if (progress % PROGRESS_REPORT_INTERVAL == 0) {
                    LOG.info(LOGKEY_PROCESSING_PARTIALLY_PROCESSEDMSG, progress);
                }
            }

            LOG.info(LOGKEY_PROCESSING_SUMMARY_PROCESSEDMSG, success);

            if (fail > 0) {
                LOG.warning(LOGKEY_PROCESSING_SUMMARY_ERRORMSG, fail);
            }

            //Persist the remaining records
            eventProducer.triggerEvent(SensorEventKey.SENSOR_DATA_IMPORTED);

            // Finish progressbar
            progressHandle.finish();
//        } catch (ParseException ex) {
//            LOG.error(LOGKEY_FAILED_TO_CREATE_DATE_FROM_FILENAME, fileToProcess.getName(), GreenseekerPreference.SENSOR_DATA_FILE_FORMAT.getValue());
//            LOG.trace(ex);
//        } 
        } catch (Exception ex) {
            LOG.error(LOGKEY_FAILED_TO_READ_FILE, fileToProcess.getName());
//            if (dataFile != null) {
//            }
            LOG.trace(ex);
        }
        return null;
    }

    /**
     * Generic greenseeker method to build a Greenseeker specific metadata object
     *
     * @return Greenseeker metadata object
     */
    @Override
    protected SensorMetadata buildMetadataFromPreferences() {
        String fileFormat = GreenseekerPreference.SENSOR_DATA_FILE_FORMAT.getValue();
        int sensorHeigth = Integer.parseInt(GreenseekerPreference.GREENSEEKER_SENSOR_HEIGTH.getValue());
        int numberOfSensors = Integer.parseInt(GreenseekerPreference.NUMBER_OF_GREENSEEKER_SENSORS.getValue());
        String greenseekerSensorSpread = GreenseekerPreference.GREENSEEKER_SENSOR_SPREAD.getValue();
        String sensorType = GreenseekerPreference.SENSOR_TYPE.getValue();
        String spectralChannels = GreenseekerPreference.SPECTRAL_CHANNELS.getValue();
        return new SensorMetadata(fileFormat, sensorHeigth, numberOfSensors, greenseekerSensorSpread, sensorType, spectralChannels);
    }

    /**
     * Generic greenseeker method to remove multiple spaces from payload String and split it into a String array
     *
     * @param payload The String to convert to an array
     * @return the splitted payload String
     */
    protected String[] getFields(String payload) {
        String payloadSpaced = payload.replaceAll("\\s{1,}", " ");
        String payloadTrimmed = payloadSpaced.trim();

        return payloadTrimmed.split(GreenseekerPreference.DELIMITER.getValue()); // Normal is leading, borne is the same
    }

    @Override
    protected String getFileDateFormat() {
        return GreenseekerPreference.SENSOR_DATA_FILE_FORMAT.getValue();
    }

    private class EventProducer extends BaseGuiEventProducer {

        public EventProducer() {
            super(SensorEventKey.class);
        }
    }
}
