/**
 * Copyright (C) 2010 Cloudfarming <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.sensor.greenseeker;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.EnumMap;
import java.util.List;
import org.openide.util.NbBundle;

/**
 *
 * @author Gerben Feenstra
 */
public class GreenseekerFileProcessorNormal extends GreenseekerFileProcessor {

    // Constants    
    private static final SimpleDateFormat TIME_FORMAT = new SimpleDateFormat("HHmmss.SS");
    private static final String LOGKEY_TYPE_DISPLAYNAME = NbBundle.getMessage(GreenseekerFileProcessorNormal.class, "sensor.greenseeker.fileprocessor.normal.log.displayname");

    public GreenseekerFileProcessorNormal(File fileToProcess) {
        super(fileToProcess);
    }

    /**
     * Parse Greenseeker file, return list of raw records.
     * @return A list of raw records
     * @throws FileNotFoundException
     * @throws IOException 
     */
    @Override
    protected List<String> parseFile(File greenseekerBorneFile) throws FileNotFoundException, IOException {
        List<String> recordList = new ArrayList<String>();

        // Create new BufferedInputStream
        BufferedInputStream bis = new BufferedInputStream(new FileInputStream(greenseekerBorneFile));

        // Create record byte[] container
        final byte[] record = new byte[GreenseekerFileDescriptionNormal.RECORD_LENGTH];

        // Skip header
        bis.skip(GreenseekerFileDescriptionNormal.HEADER_LENGTH);

        // Fill list
        while (bis.read(record) > 0) {
            String rec = new String(record);

            recordList.add(rec);
        }
        return recordList;
    }

    /**
     * Parses one record of a normal Greenseeker sensorfile.
     *
     * @param payload The record to be processed.
     * @param dataFile The SensorDataFile belonging to this sensor file.
     * @return A record represented as a Greenseeker sensordata object
     * @throws ParseException Exception occurs when date/time cannot be parsed.
     */
    @Override
    protected GreenseekerSensorData parseLine(String payload) throws GreenseekerParseException {

        // Split payload into fields-array
        String[] fields = getFields(payload);

        // First create a entity representation of a file
        //converting the differrent data
        EnumMap map = new EnumMap<GreenseekerFileDescriptionNormal, Object>(GreenseekerFileDescriptionNormal.class);
        try {
            for (GreenseekerFileDescriptionNormal greenseekerNormalField : GreenseekerFileDescriptionNormal.values()) {
                if (greenseekerNormalField.isMandatory()) {
                    if (greenseekerNormalField.getTheClass().equals(java.lang.Double.class)) {
                        double theDouble = Double.parseDouble(fields[greenseekerNormalField.getPosition()]);
                        map.put(greenseekerNormalField, theDouble);
                    } else if (greenseekerNormalField.getTheClass().equals(java.lang.Integer.class)) {
                        int theInt = Integer.parseInt(fields[greenseekerNormalField.getPosition()]);
                        map.put(greenseekerNormalField, theInt);
                    } else if (greenseekerNormalField.getTheClass().equals(java.util.Date.class)) {
                        Date theDate = TIME_FORMAT.parse(fields[greenseekerNormalField.getPosition()]);
                        map.put(greenseekerNormalField, theDate);
                    } else {
                        map.put(greenseekerNormalField, fields[greenseekerNormalField.getPosition()]);
                    }
                }
            }
        } catch (ParseException e) {
            throw new GreenseekerParseException("Error parsing date ", e);
        } catch (Exception e) {
            throw new GreenseekerParseException(e);
        }

        GreenseekerSensorData sensorData = new GreenseekerSensorData();

        sensorData.setObjectId((Double) map.get(GreenseekerFileDescriptionNormal.OBJECTID));
        sensorData.setLongitude((Double) map.get(GreenseekerFileDescriptionNormal.LONGITUDE));
        sensorData.setLatitude((Double) map.get(GreenseekerFileDescriptionNormal.LATITUDE));
        sensorData.setSpeed((Double) map.get(GreenseekerFileDescriptionNormal.SPEED));
        sensorData.setHeading((Double) map.get(GreenseekerFileDescriptionNormal.HEADING));
        sensorData.setElevation((Double) map.get(GreenseekerFileDescriptionNormal.ELEVATION));
        sensorData.setNdvi((Double) map.get(GreenseekerFileDescriptionNormal.NDVI));
        sensorData.setVi2nd((Double) map.get(GreenseekerFileDescriptionNormal.VI_2ND));

        sensorData.setSensor10((Double) map.get(GreenseekerFileDescriptionNormal.SENSOR_10));
        sensorData.setSensor19((Double) map.get(GreenseekerFileDescriptionNormal.SENSOR_19));
        sensorData.setSensor23((Double) map.get(GreenseekerFileDescriptionNormal.SENSOR_23));
        sensorData.setSensor24((Double) map.get(GreenseekerFileDescriptionNormal.SENSOR_24));
        sensorData.setSensor26((Double) map.get(GreenseekerFileDescriptionNormal.SENSOR_26));
        sensorData.setSensor73((Double) map.get(GreenseekerFileDescriptionNormal.SENSOR_73));

        return sensorData;
    }

    /**
     * Concrete subclasses must override this method to return the displayname of that sensorprocessor to be displayed in the log.
     * For example: "ASC-340 Cropcircle"
     * 
     * Primary use is for logging, for example: [Info] Start processing of ASC-340 Cropcircle sensor file 010710AD.CSV
     * 
     * @return 
     */
    @Override
    protected String getLogDisplayName() {
        return LOGKEY_TYPE_DISPLAYNAME;
    }
}
